#version 330

in vec4 vertexOutTextcoord[];
in vec4 vertexOutColor[];
in vec3 vertexOutNormal[];
in vec3 vertexOutPosition[];

out vec4 transformFeedbackTextcoord;
out vec4 transformFeedbackColor;
out vec3 transformFeedbackNormal;
out vec4 transformFeedbackPosition;
flat out vec4 varaColor;

uniform int shaderModeSwitch;
uniform float time;
uniform vec3 shipPosition;
uniform vec3 shipFlightVector;
uniform float timeBetweenGeneratingVertices;
uniform vec4 randomNumber;
uniform float particleOldAge;

layout (points) in;
layout (points) out;
layout (max_vertices = 3) out;

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main(void)
{
    // state:
    // 5.0 => vertex, that is off gas, counting it's life time and moving around
    // -5.0 => vertex, that is generating other vertices and should be invisible
    float state = vertexOutTextcoord[0].y;
    vec3 position = vertexOutPosition[0];
    vec3 flyDirection = vertexOutNormal[0];
    float flyingTime = vertexOutTextcoord[0].x;

    if(shaderModeSwitch==1) {
        // drawing
        // discard generator verices
        if(state > 0) {
            varaColor = vertexOutColor[0];
            gl_Position = gl_in[0].gl_Position;
            gl_PointSize = gl_in[0].gl_PointSize;
            EmitVertex();
            EndPrimitive();
        }
        else {
            // debug output generating vertices
//             varaColor = vec4(0, flyingTime/10., flyingTime/60., 1);
//             gl_Position = gl_in[0].gl_Position;
//             gl_PointSize = gl_in[0].gl_PointSize;
//             EmitVertex();
//             EndPrimitive();
        }
    }
    else {
        // feedback buffer


        if(state > 0) {
            if(flyingTime < particleOldAge) {

                // forward non generating vertices with updated time
                transformFeedbackTextcoord = vec4(flyingTime+time, vertexOutTextcoord[0].yzw);
                transformFeedbackColor = vertexOutColor[0];
                transformFeedbackNormal = vertexOutNormal[0];
                transformFeedbackPosition = vec4(vertexOutPosition[0], 1);
                EmitVertex();
                EndPrimitive();
            }
        }
        else /*if (state < 0)*/ {
            // if the timer timed out, generate a vertex
            bool generateNewVertex = false;
            float lifeTime = vertexOutTextcoord[0].w+time;
            if(flyingTime >= timeBetweenGeneratingVertices && lifeTime < 5) {
                generateNewVertex = true;
                //reset flyingTime of generated vertex, so that the counter starts again
                flyingTime = 0;
            }

            // forward generating vertex with updated time counter
            transformFeedbackTextcoord = vec4(flyingTime+time, -5, 0, lifeTime);
            transformFeedbackColor = vertexOutColor[0];
            transformFeedbackNormal = vertexOutNormal[0];
            transformFeedbackPosition = vec4(vertexOutPosition[0], 1);
            EmitVertex();
            EndPrimitive();

            if(generateNewVertex) {
                transformFeedbackTextcoord = vec4(lifeTime/3.0, 5, 0, randomNumber.x*30+100*lifeTime);   //vec4(lifetime, state, direction change timer, size )
                transformFeedbackColor = vec4(1, 1, 1, 1);
                transformFeedbackNormal = vec3(0, 0, 0);   //fly direction
                transformFeedbackPosition = vec4(shipPosition + vertexOutPosition[0], 1);
                EmitVertex();
                EndPrimitive();
            }
        }
    }
}
