#version 330

in vec4 vertexTexCoords;
in vec4 vertexColor;
in vec3 vertexNormal;
in vec3 vertexPosition;
uniform int framebufferHeight;
uniform int framebufferWidth;
uniform int shaderModeSwitch;
uniform mat4 modelViewProjectionMatrix;
uniform vec4 randomNumber;
uniform vec3 cameraPosition;
uniform float time;

out vec4 vertexOutTextcoord;
out vec4 vertexOutColor;
out vec3 vertexOutNormal;
out vec3 vertexOutPosition;

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main(void) {
    if(shaderModeSwitch==1) {
        // drawing
        vec4 position = modelViewProjectionMatrix * vec4(vertexPosition, 1);
        float worldCoordSize = vertexTexCoords.w;
//         vec3 vecPointingToCamera = normalize(cameraPosition - vertexPosition);
//         vec3 parallelToMonitor = cross(vecPointingToCamera, vecPointingToCamera.yzx);
//
//         vec4 sizeTmp = modelViewProjectionMatrix * vec4(vertexPosition + parallelToMonitor, 1);
//         float sizeTmp2 = length(vec2(sizeTmp - position));
//         float size = 60/position.z;
        float size = ((float(framebufferHeight)/600.0) * worldCoordSize/position.z);
//         float size = (((float(framebufferHeight))/600.)) * (worldCoordSize*sizeTmp2/position.z);
//         float size = sizeTmp.x;

        float alpha = 150/worldCoordSize;
        alpha /= 2.0;
        if(alpha>1)
            alpha = 1;

//         if(size < 6)
//             alpha = ((size-2)/4.);
//
//         if(size <= 2)
//             alpha = 0;

        vertexOutTextcoord = vertexTexCoords;
        vertexOutColor = vertexColor*vec4(1, 1, 1, alpha);
        vertexOutNormal = vertexNormal;
        vertexOutPosition = vec3(position);

        gl_Position = position;
        gl_PointSize = size;
    }
    else {
        // feedback buffer
        vec3 flyDirection = vertexNormal;
        float flyingTime = vertexTexCoords.x;
        float state = vertexTexCoords.y;
        float directionChangeTimer = vertexTexCoords.z;
        float size = vertexTexCoords.w;
        // state:
        // 5.0 => vertex, that is off gas, counting it's life time and moving around
        // -5.0 => vertex, that is generating other vertices and should be invisible

//         if(directionChangeCountDown<0) {
//             directionChangeCountDown = randomNumber.a*10;
        //             flyDirection = randomNumber.xyz*2-vec3(1);
//         }
        if(state > 0) {
            // off gas vertex
//             flyDirection = flyDirection * (1-time) + randomNumber.xyz * time;
            float randomiseFactor = 0.70;
            if(flyingTime < 0.7) {
                flyDirection = flyDirection * (1-time);
                randomiseFactor = 0.01;
//                 size = size + 30*time;
            }
            else if(flyingTime < 1.2) {
                flyDirection = flyDirection * (1-time);
                randomiseFactor = 0.4;
//                 size = size + 60*time;
            }
            size = size + 100*time*flyingTime;

//             else {
                directionChangeTimer += time;
                if(directionChangeTimer > randomNumber.w*5.) {
    //                 vec3 tmp = randomNumber.xyz;
                    vec3 tmp = vec3(rand(vertexPosition.xy), rand(vertexPosition.yz), rand(vertexPosition.zx));
                    tmp = tmp*2 - vec3(1);
                    tmp = tmp*(2+2*flyingTime);
                    flyDirection = flyDirection * (1.0-randomiseFactor) + tmp * randomiseFactor;
                    directionChangeTimer = 0;
                }
//             }



            vec4 outColor;
            // the vertex starts white, then yellow, orange, red and grey
            //       until [sec]:  0.6,         1.4,    2.3, 3.3,    above
            if (flyingTime<0.1)
                outColor = vec4(1, 1, 1, 1);
            else if (flyingTime < 0.2)
                outColor = vec4(1, 1, 0, 1);
            else if (flyingTime < 0.5)
                outColor = vec4(1, 0.5, 0, 1);
            else if (flyingTime < 0.7)
                outColor = vec4(1, 0, 0, 1);
            else
                outColor = vec4(0.20, 0.15, 0.15, 1);

            vec3 newPosition = vertexPosition + flyDirection*time*0.5;
            if(newPosition.x>100) newPosition.x=-100;
            if(newPosition.y>100) newPosition.y=-100;
            if(newPosition.z>100) newPosition.z=-100;
            if(newPosition.x<-100) newPosition.x=100;
            if(newPosition.y<-100) newPosition.y=100;
            if(newPosition.z<-100) newPosition.z=100;

            vertexOutTextcoord = vec4(flyingTime, state, directionChangeTimer, size);
            vertexOutColor = outColor;
            vertexOutNormal = flyDirection;
            vertexOutPosition = newPosition;
        }
        else {
            // generator vertex
            vertexOutTextcoord = vertexTexCoords;
            vertexOutColor = vertexColor;
            vertexOutNormal = vertexNormal;
            vertexOutPosition = vertexPosition;
        }
    }
}